from typing import Any

import rlviser_py as rlviser
from rlgym.api.config.renderer import Renderer
from rlgym.rocket_league.api.game_state import GameState
from rlgym.rocket_league.common_values import BOOST_LOCATIONS


class RLViserRenderer(Renderer[GameState]):

    def __init__(self, tick_rate=120/8):
        rlviser.set_boost_pad_locations(BOOST_LOCATIONS)
        self.tick_rate = tick_rate
        self.packet_id = 0

    def render(self, state: GameState, _) -> Any:
        self.packet_id += 1
        rlviser.render_rlgym(self.packet_id, self.tick_rate, state)

    def close(self):
        rlviser.quit()



from typing import List, Tuple

def set_boost_pad_locations(locations: List[Tuple[float, float, float]]) -> ...:
    pass

try:
    from RocketSim import BallState, CarConfig, CarState, Team

    def render(tick_count: int, tick_rate: float, boost_pad_states: List[bool], ball: BallState, cars: List[Tuple[int, Team, CarConfig, CarState]]) -> ...:
        pass
except ImportError:
    pass

try:
    from rlgym.rocket_league.engine.game_state import GameState

    def render_rlgym(tick_count: int, tick_rate: float, gym_state: GameState) -> ...:
        pass
except ImportError:
    pass

def quit() -> ...:
    pass



import time
import numpy as np
from itertools import chain

from rlgym.api import RLGym
from rlgym.rocket_league.action_parsers import LookupTableAction, RepeatAction
from rlgym.rocket_league.done_conditions import GoalCondition, AnyCondition, TimeoutCondition, NoTouchTimeoutCondition
from rlgym.rocket_league.obs_builders import DefaultObs
from rlgym.rocket_league.reward_functions import CombinedReward, GoalReward, TouchReward
from rlgym.rocket_league.sim import RocketSimEngine
from rlgym.rocket_league.state_mutators import MutatorSequence, FixedTeamSizeMutator, KickoffMutator

from gym_renderer import RLViserRenderer

env = RLGym(
    state_mutator=MutatorSequence(
        FixedTeamSizeMutator(blue_size=2, orange_size=2),
        KickoffMutator()
    ),
    obs_builder=DefaultObs(zero_padding=None),
    action_parser=RepeatAction(LookupTableAction(), repeats=8),
    reward_fn=CombinedReward(
        (GoalReward(), 10.),
        (TouchReward(), 0.1)
    ),
    termination_cond=GoalCondition(),
    truncation_cond=AnyCondition(
        TimeoutCondition(timeout=300.),
        NoTouchTimeoutCondition(timeout=30.)
    ),
    transition_engine=RocketSimEngine(),
    renderer=RLViserRenderer()
)

# simulate 2 episodes
for _ in range(2):
    obs_dict = env.reset()
    steps = 0
    ep_reward = {agent_id: 0. for agent_id in env.agents}
    t0 = time.time()
    while True:
        env.render()

        actions = {}
        for agent_id, action_space in env.action_spaces.items():
            # agent.act(obs) | Your agent should go here
            actions[agent_id] = np.random.randint(action_space, size=(1,))

        obs_dict, reward_dict, terminated_dict, truncated_dict = env.step(actions)
        for agent_id, reward in reward_dict.items():
            ep_reward[agent_id] += reward

        if any(chain(terminated_dict.values(), truncated_dict.values())):
            break

        time.sleep(max(0, t0 + steps / 15 - time.time()))
        steps += 1

    ep_time = time.time() - t0
    print("Steps per second: {:.0f} | Episode time: {:.2f} | Episode Reward: {:.2f}".format(
        steps / ep_time, ep_time, max(ep_reward.values())))
